@@ -547,6 +547,32 @@ ply_boot_client_update_daemon (ply_boot_client_t                  *client,
                                  status, handler, failed_handler, user_data);
 }
 
+void
+ply_boot_client_change_mode (ply_boot_client_t                  *client,
+                             const char                         *new_mode,
+                             ply_boot_client_response_handler_t  handler,
+                             ply_boot_client_response_handler_t  failed_handler,
+                             void                               *user_data)
+{
+  assert (client != NULL);
+
+  ply_boot_client_queue_request (client, PLY_BOOT_PROTOCOL_REQUEST_TYPE_CHANGE_MODE,
+                                 new_mode, handler, failed_handler, user_data);
+}
+
+void
+ply_boot_client_system_update (ply_boot_client_t                  *client,
+                               const char                         *progress,
+                               ply_boot_client_response_handler_t  handler,
+                               ply_boot_client_response_handler_t  failed_handler,
+                               void                               *user_data)
+{
+  assert (client != NULL);
+
+  ply_boot_client_queue_request (client, PLY_BOOT_PROTOCOL_REQUEST_TYPE_SYSTEM_UPDATE,
+                                 progress, handler, failed_handler, user_data);
+}
+
 void
 ply_boot_client_tell_daemon_to_change_root (ply_boot_client_t                  *client,
                                             const char                         *root_dir,
@@ -843,137 +869,4 @@ ply_boot_client_attach_to_event_loop (ply_boot_client_t *client,
 
 }
 
-#ifdef PLY_BOOT_CLIENT_ENABLE_TEST
-
-#include <stdio.h>
-
-#include "ply-event-loop.h"
-#include "ply-boot-client.h"
-
-static void
-on_pinged (ply_event_loop_t *loop)
-{
-  printf ("PING!\n");
-}
-
-static void
-on_ping_failed (ply_event_loop_t *loop)
-{
-  printf ("PING FAILED! %m\n");
-  ply_event_loop_exit (loop, 1);
-}
-
-static void
-on_update (ply_event_loop_t *loop)
-{
-  printf ("UPDATE!\n");
-}
-
-static void
-on_update_failed (ply_event_loop_t *loop)
-{
-  printf ("UPDATE FAILED! %m\n");
-  ply_event_loop_exit (loop, 1);
-}
-
-static void
-on_newroot (ply_event_loop_t *loop)
-{
-  printf ("NEWROOT!\n");
-}
-
-static void
-on_system_initialized (ply_event_loop_t *loop)
-{
-  printf ("SYSTEM INITIALIZED!\n");
-}
-
-static void
-on_system_initialized_failed (ply_event_loop_t *loop)
-{
-  printf ("SYSTEM INITIALIZATION REQUEST FAILED!\n");
-  ply_event_loop_exit (loop, 1);
-}
-
-static void
-on_quit (ply_event_loop_t *loop)
-{
-  printf ("QUIT!\n");
-  ply_event_loop_exit (loop, 0);
-}
-
-static void
-on_quit_failed (ply_event_loop_t *loop)
-{
-  printf ("QUIT FAILED! %m\n");
-  ply_event_loop_exit (loop, 2);
-}
-
-static void
-on_disconnect (ply_event_loop_t *loop)
-{
-  printf ("DISCONNECT!\n");
-  ply_event_loop_exit (loop, 1);
-}
-
-int
-main (int    argc,
-      char **argv)
-{
-  ply_event_loop_t *loop;
-  ply_boot_client_t *client;
-  int exit_code;
-
-  exit_code = 0;
-
-  loop = ply_event_loop_new ();
-
-  client = ply_boot_client_new ();
-
-  if (!ply_boot_client_connect (client, 
-                                (ply_boot_client_disconnect_handler_t) on_disconnect,
-                                loop))
-    {
-      perror ("could not start boot client");
-      return errno;
-    }
-
-  ply_boot_client_attach_to_event_loop (client, loop);
-  ply_boot_client_ping_daemon (client, 
-                               (ply_boot_client_response_handler_t) on_pinged,
-                               (ply_boot_client_response_handler_t) on_ping_failed,
-                               loop);
-
-  ply_boot_client_update_daemon (client, 
-                                 "loading",
-                                 (ply_boot_client_response_handler_t) on_update,
-                                 (ply_boot_client_response_handler_t) on_update_failed,
-                                 loop);
-
-  ply_boot_client_update_daemon (client, 
-                                 "loading more",
-                                 (ply_boot_client_response_handler_t) on_update,
-                                 (ply_boot_client_response_handler_t) on_update_failed,
-                                 loop);
-
-  ply_boot_client_tell_daemon_system_is_initialized (client, 
-                                       (ply_boot_client_response_handler_t) 
-                                       on_system_initialized,
-                                       (ply_boot_client_response_handler_t) 
-                                       on_system_initialized_failed,
-                                       loop);
-
-  ply_boot_client_tell_daemon_to_quit (client, 
-                                       (ply_boot_client_response_handler_t) on_quit,
-                                       (ply_boot_client_response_handler_t) on_quit_failed,
-                                       loop);
-
-  exit_code = ply_event_loop_run (loop);
-
-  ply_boot_client_free (client);
-
-  return exit_code;
-}
-
-#endif /* PLY_BOOT_CLIENT_ENABLE_TEST */
 /* vim: set ts=4 sw=4 expandtab autoindent cindent cino={.5s,(0: */
