@@ -48,21 +48,16 @@
 #define UPDATES_PER_SECOND 30
 #endif
 
-#define KEY_CTRL_L ('\100' ^'L')
-#define KEY_CTRL_T ('\100' ^'T')
-#define KEY_CTRL_V ('\100' ^'V')
-
 struct _ply_boot_splash
 {
   ply_event_loop_t *loop;
   ply_module_handle_t *module_handle;
   const ply_boot_splash_plugin_interface_t *plugin_interface;
   ply_boot_splash_plugin_t *plugin;
-  ply_keyboard_t *keyboard;
+  ply_boot_splash_mode_t mode;
   ply_buffer_t *boot_buffer;
   ply_trigger_t *idle_trigger;
-  ply_list_t *pixel_displays;
-  ply_list_t *text_displays;
+  ply_list_t *seats;
 
   char *theme_path;
   char *plugin_dir;
@@ -73,7 +68,6 @@ struct _ply_boot_splash
   void *idle_handler_user_data;
 
   uint32_t is_loaded : 1;
-  uint32_t is_shown : 1;
   uint32_t should_force_text_mode : 1;
 };
 
@@ -97,21 +91,33 @@ ply_boot_splash_new (const char     *theme_path,
   splash->theme_path = strdup (theme_path);
   splash->plugin_dir = strdup (plugin_dir);
   splash->module_handle = NULL;
-  splash->is_shown = false;
+  splash->mode = PLY_BOOT_SPLASH_MODE_INVALID;
 
   splash->boot_buffer = boot_buffer;
-  splash->pixel_displays = ply_list_new ();
-  splash->text_displays = ply_list_new ();
+  splash->seats = ply_list_new ();
 
   return splash;
 }
 
 static void
-refresh_displays (ply_boot_splash_t *splash)
+detach_from_seat (ply_boot_splash_t *splash,
+                  ply_seat_t        *seat)
 {
-  ply_list_node_t *node;
+  ply_keyboard_t *keyboard;
+  ply_list_t *displays;
+  ply_list_node_t *node, *next_node;
 
-  node = ply_list_get_first_node (splash->pixel_displays);
+  ply_trace ("removing keyboard");
+  if (splash->plugin_interface->unset_keyboard != NULL)
+    {
+      keyboard = ply_seat_get_keyboard (seat);
+      splash->plugin_interface->unset_keyboard (splash->plugin, keyboard);
+    }
+
+  ply_trace ("removing pixel displays");
+  displays = ply_seat_get_pixel_displays (seat);
+
+  node = ply_list_get_first_node (displays);
   while (node != NULL)
     {
       ply_pixel_display_t *display;
@@ -119,184 +125,137 @@ refresh_displays (ply_boot_splash_t *splash)
       unsigned long width, height;
 
       display = ply_list_node_get_data (node);
-      next_node = ply_list_get_next_node (splash->pixel_displays, node);
+      next_node = ply_list_get_next_node (displays, node);
 
       width = ply_pixel_display_get_width (display);
       height = ply_pixel_display_get_height (display);
 
-      ply_pixel_display_draw_area (display, 0, 0, width, height);
+      ply_trace ("Removing %lux%lu pixel display", width, height);
+
+      if (splash->plugin_interface->remove_pixel_display != NULL)
+        splash->plugin_interface->remove_pixel_display (splash->plugin, display);
+
       node = next_node;
     }
 
-  node = ply_list_get_first_node (splash->text_displays);
+  ply_trace ("removing text displays");
+  displays = ply_seat_get_text_displays (seat);
+
+  node = ply_list_get_first_node (displays);
   while (node != NULL)
     {
       ply_text_display_t *display;
-      ply_list_node_t *next_node;
       int number_of_columns, number_of_rows;
 
       display = ply_list_node_get_data (node);
-      next_node = ply_list_get_next_node (splash->text_displays, node);
+      next_node = ply_list_get_next_node (displays, node);
 
       number_of_columns = ply_text_display_get_number_of_columns (display);
       number_of_rows = ply_text_display_get_number_of_rows (display);
 
-      ply_text_display_draw_area (display, 0, 0,
-                                  number_of_columns,
-                                  number_of_rows);
-      node = next_node;
-    }
-}
+      ply_trace ("Removing %dx%d text display", number_of_columns, number_of_rows);
 
-static ply_terminal_t *
-find_local_console_terminal (ply_boot_splash_t *splash)
-{
-  ply_list_node_t *node;
-  node = ply_list_get_first_node (splash->text_displays);
-
-  while (node != NULL)
-    {
-      ply_text_display_t *display;
-      ply_terminal_t *terminal;
-      ply_list_node_t *next_node;
-
-      display = ply_list_node_get_data (node);
-      next_node = ply_list_get_next_node (splash->text_displays, node);
-
-      terminal = ply_text_display_get_terminal (display);
-
-      if (terminal != NULL && ply_terminal_is_vt (terminal))
-        return terminal;
+      if (splash->plugin_interface->remove_text_display != NULL)
+        splash->plugin_interface->remove_text_display (splash->plugin, display);
 
       node = next_node;
     }
-
-  return NULL;
 }
 
 static void
-on_keyboard_input (ply_boot_splash_t *splash,
-                   const char        *keyboard_input,
-                   size_t             character_size)
+attach_to_seat (ply_boot_splash_t *splash,
+                ply_seat_t        *seat)
 {
-  wchar_t key;
+  ply_keyboard_t *keyboard;
+  ply_list_t *displays;
+  ply_list_node_t *node, *next_node;
 
-  if ((ssize_t) mbrtowc (&key, keyboard_input, character_size, NULL) > 0)
+  if (splash->plugin_interface->set_keyboard != NULL)
     {
-      switch (key)
+      keyboard = ply_seat_get_keyboard (seat);
+      splash->plugin_interface->set_keyboard (splash->plugin, keyboard);
+    }
+
+  if (splash->plugin_interface->add_pixel_display != NULL)
+    {
+      displays = ply_seat_get_pixel_displays (seat);
+
+      ply_trace ("adding pixel displays");
+      node = ply_list_get_first_node (displays);
+      while (node != NULL)
         {
-          case KEY_CTRL_L:
-            refresh_displays (splash);
-          return;
+          ply_pixel_display_t *display;
+          ply_list_node_t *next_node;
+          unsigned long width, height;
 
-          case KEY_CTRL_T:
-            ply_trace ("toggle text mode!");
-            splash->should_force_text_mode = !splash->should_force_text_mode;
+          display = ply_list_node_get_data (node);
+          next_node = ply_list_get_next_node (displays, node);
 
-            if (ply_list_get_length (splash->pixel_displays) >= 1)
-              {
-                ply_terminal_t *terminal;
+          width = ply_pixel_display_get_width (display);
+          height = ply_pixel_display_get_height (display);
 
-                terminal = find_local_console_terminal (splash);
+          ply_trace ("Adding %lux%lu pixel display", width, height);
 
-                if (terminal != NULL)
-                  {
-                    if (splash->should_force_text_mode)
-                      {
-                        ply_terminal_set_mode (terminal, PLY_TERMINAL_MODE_TEXT);
-                        ply_terminal_ignore_mode_changes (terminal, true);
-                      }
-                    else
-                      ply_terminal_ignore_mode_changes (terminal, false);
-                  }
-              }
-            ply_trace ("text mode toggled!");
-          return;
+          splash->plugin_interface->add_pixel_display (splash->plugin, display);
 
-          case KEY_CTRL_V:
-            ply_trace ("toggle verbose mode!");
-            ply_toggle_tracing ();
-            ply_trace ("verbose mode toggled!");
-          return;
+          node = next_node;
+        }
+    }
+
+  if (splash->plugin_interface->add_text_display != NULL)
+    {
+      displays = ply_seat_get_text_displays (seat);
+
+      ply_trace ("adding text displays");
+      node = ply_list_get_first_node (displays);
+      while (node != NULL)
+        {
+          ply_text_display_t *display;
+          int number_of_columns, number_of_rows;
+
+          display = ply_list_node_get_data (node);
+          next_node = ply_list_get_next_node (displays, node);
+
+          number_of_columns = ply_text_display_get_number_of_columns (display);
+          number_of_rows = ply_text_display_get_number_of_rows (display);
+
+          ply_trace ("Adding %dx%d text display", number_of_columns, number_of_rows);
+
+          splash->plugin_interface->add_text_display (splash->plugin, display);
+
+          node = next_node;
         }
     }
 }
 
 void
-ply_boot_splash_set_keyboard (ply_boot_splash_t *splash,
-                              ply_keyboard_t    *keyboard)
+ply_boot_splash_attach_to_seat (ply_boot_splash_t *splash,
+                                ply_seat_t        *seat)
 {
-  splash->keyboard = keyboard;
+  ply_list_node_t *node;
 
-  ply_keyboard_add_input_handler (keyboard,
-                                  (ply_keyboard_input_handler_t)
-                                  on_keyboard_input, splash);
+  node = ply_list_find_node (splash->seats, seat);
 
-  if (splash->plugin_interface->set_keyboard == NULL)
+  if (node != NULL)
     return;
 
-  splash->plugin_interface->set_keyboard (splash->plugin, keyboard);
+  ply_list_append_data (splash->seats, seat);
+  attach_to_seat (splash, seat);
 }
 
 void
-ply_boot_splash_unset_keyboard (ply_boot_splash_t *splash)
+ply_boot_splash_detach_from_seat (ply_boot_splash_t *splash,
+                                  ply_seat_t        *seat)
 {
-  ply_keyboard_remove_input_handler (splash->keyboard,
-                                     (ply_keyboard_input_handler_t)
-                                     on_keyboard_input);
+  ply_list_node_t *node;
 
-  if (splash->plugin_interface->set_keyboard == NULL)
+  node = ply_list_find_node (splash->seats, seat);
+
+  if (node == NULL)
     return;
 
-  splash->plugin_interface->unset_keyboard (splash->plugin, splash->keyboard);
-}
-
-void
-ply_boot_splash_add_pixel_display (ply_boot_splash_t   *splash,
-                                   ply_pixel_display_t *display)
-{
-  ply_list_append_data (splash->pixel_displays, display);
-
-  if (splash->plugin_interface->add_pixel_display == NULL)
-    return;
-
-  splash->plugin_interface->add_pixel_display (splash->plugin, display);
-}
-
-void
-ply_boot_splash_remove_pixel_display (ply_boot_splash_t   *splash,
-                                      ply_pixel_display_t *display)
-{
-  ply_list_remove_data (splash->pixel_displays, display);
-
-  if (splash->plugin_interface->remove_pixel_display == NULL)
-    return;
-
-  splash->plugin_interface->remove_pixel_display (splash->plugin, display);
-}
-
-void
-ply_boot_splash_add_text_display (ply_boot_splash_t   *splash,
-                                  ply_text_display_t *display)
-{
-  ply_list_append_data (splash->text_displays, display);
-
-  if (splash->plugin_interface->add_text_display == NULL)
-    return;
-
-  splash->plugin_interface->add_text_display (splash->plugin, display);
-}
-
-void
-ply_boot_splash_remove_text_display (ply_boot_splash_t   *splash,
-                                     ply_text_display_t *display)
-{
-  ply_list_remove_data (splash->text_displays, display);
-
-  if (splash->plugin_interface->remove_pixel_display == NULL)
-    return;
-
-  splash->plugin_interface->remove_text_display (splash->plugin, display);
+  ply_list_remove_data (splash->seats, seat);
+  detach_from_seat (splash, seat);
 }
 
 bool
@@ -432,56 +391,24 @@ ply_boot_splash_unload (ply_boot_splash_t *splash)
 }
 
 static void
-remove_displays (ply_boot_splash_t *splash)
+detach_from_seats (ply_boot_splash_t *splash)
 {
-  ply_list_node_t *node, *next_node;
+  ply_list_node_t *node;
 
-  ply_trace ("removing pixel displays");
+  ply_trace ("detaching from seats");
 
-  node = ply_list_get_first_node (splash->pixel_displays);
+  node = ply_list_get_first_node (splash->seats);
   while (node != NULL)
     {
-      ply_pixel_display_t *display;
+      ply_seat_t *seat;
       ply_list_node_t *next_node;
-      unsigned long width, height;
 
-      display = ply_list_node_get_data (node);
-      next_node = ply_list_get_next_node (splash->pixel_displays, node);
+      seat = ply_list_node_get_data (node);
+      next_node = ply_list_get_next_node (splash->seats, node);
 
-      width = ply_pixel_display_get_width (display);
-      height = ply_pixel_display_get_height (display);
+      detach_from_seat (splash, seat);
 
-      ply_trace ("Removing %lux%lu pixel display", width, height);
-
-      if (splash->plugin_interface->remove_pixel_display != NULL)
-        splash->plugin_interface->remove_pixel_display (splash->plugin, display);
-
-      ply_trace ("Removing node");
-      ply_list_remove_node (splash->pixel_displays, node);
-
-      node = next_node;
-    }
-
-  ply_trace ("removing text displays");
-  node = ply_list_get_first_node (splash->text_displays);
-  while (node != NULL)
-    {
-      ply_text_display_t *display;
-      int number_of_columns, number_of_rows;
-
-      display = ply_list_node_get_data (node);
-      next_node = ply_list_get_next_node (splash->text_displays, node);
-
-      number_of_columns = ply_text_display_get_number_of_columns (display);
-      number_of_rows = ply_text_display_get_number_of_rows (display);
-
-      ply_trace ("Removing %dx%d text display", number_of_columns, number_of_rows);
-
-      if (splash->plugin_interface->remove_text_display != NULL)
-        splash->plugin_interface->remove_text_display (splash->plugin, display);
-
-      ply_trace ("Removing node");
-      ply_list_remove_node (splash->text_displays, node);
+      ply_list_remove_node (splash->seats, node);
 
       node = next_node;
     }
@@ -508,9 +435,8 @@ ply_boot_splash_free (ply_boot_splash_t *splash)
                                              splash);
     }
 
-  remove_displays (splash);
-  ply_list_free (splash->pixel_displays);
-  ply_list_free (splash->text_displays);
+  detach_from_seats (splash);
+  ply_list_free (splash->seats);
 
   if (splash->module_handle != NULL)
     ply_boot_splash_unload (splash);
@@ -564,17 +490,25 @@ ply_boot_splash_show (ply_boot_splash_t *splash,
                       ply_boot_splash_mode_t mode)
 {
   assert (splash != NULL);
+  assert (mode != PLY_BOOT_SPLASH_MODE_INVALID);
   assert (splash->module_handle != NULL);
   assert (splash->loop != NULL);
-
-  if (splash->is_shown)
-    return true;
-
   assert (splash->plugin_interface != NULL);
   assert (splash->plugin != NULL);
   assert (splash->plugin_interface->show_splash_screen != NULL);
 
-  ply_trace ("showing splash screen\n");
+  if (splash->mode == mode)
+    {
+      ply_trace ("already set same splash screen mode");
+      return true;
+    }
+  else if (splash->mode != PLY_BOOT_SPLASH_MODE_INVALID)
+    {
+      splash->plugin_interface->hide_splash_screen (splash->plugin,
+                                                    splash->loop);
+    }
+
+  ply_trace ("showing splash screen");
   if (!splash->plugin_interface->show_splash_screen (splash->plugin,
                                                      splash->loop,
                                                      splash->boot_buffer,
@@ -592,7 +526,26 @@ ply_boot_splash_show (ply_boot_splash_t *splash,
       ply_boot_splash_update_progress (splash);
     }
 
-  splash->is_shown = true;
+  splash->mode = mode;
+  return true;
+}
+
+bool
+ply_boot_splash_system_update (ply_boot_splash_t *splash,
+                               int                progress)
+{
+  assert (splash != NULL);
+  assert (splash->module_handle != NULL);
+  assert (splash->loop != NULL);
+  assert (splash->plugin_interface != NULL);
+  assert (splash->plugin != NULL);
+
+  if (splash->plugin_interface->system_update == NULL)
+    return false;
+
+  ply_trace ("updating system %i%%", progress);
+  splash->plugin_interface->system_update (splash->plugin,
+                                           progress);
   return true;
 }
 
@@ -605,7 +558,7 @@ ply_boot_splash_update_status (ply_boot_splash_t *splash,
   assert (splash->plugin_interface != NULL);
   assert (splash->plugin != NULL);
   assert (splash->plugin_interface->update_status != NULL);
-  assert (splash->is_shown);
+  assert (splash->mode != PLY_BOOT_SPLASH_MODE_INVALID);
 
   splash->plugin_interface->update_status (splash->plugin, status);
 }
@@ -649,17 +602,7 @@ ply_boot_splash_hide (ply_boot_splash_t *splash)
   splash->plugin_interface->hide_splash_screen (splash->plugin,
                                                 splash->loop);
 
-  if (ply_list_get_length (splash->pixel_displays) >= 1)
-    {
-      ply_terminal_t *terminal;
-
-      terminal = find_local_console_terminal (splash);
-
-      if (terminal != NULL)
-        ply_terminal_set_mode (terminal, PLY_TERMINAL_MODE_TEXT);
-    }
-
-  splash->is_shown = false;
+  splash->mode = PLY_BOOT_SPLASH_MODE_INVALID;
 
   if (splash->loop != NULL)
     {
@@ -682,6 +625,7 @@ void ply_boot_splash_display_message (ply_boot_splash_t             *splash,
   assert (splash != NULL);
   assert (splash->plugin_interface != NULL);
   assert (splash->plugin != NULL);
+
   if (splash->plugin_interface->display_message != NULL)
     splash->plugin_interface->display_message (splash->plugin, message);
 }
@@ -792,148 +736,4 @@ ply_boot_splash_become_idle (ply_boot_splash_t                  *splash,
   splash->plugin_interface->become_idle (splash->plugin, splash->idle_trigger);
 }
 
-#ifdef PLY_BOOT_SPLASH_ENABLE_TEST
-
-#include <stdio.h>
-
-#include "ply-event-loop.h"
-#include "ply-boot-splash.h"
-
-typedef struct test_state test_state_t;
-struct test_state {
-  ply_event_loop_t *loop;
-  ply_boot_splash_t *splash;
-  ply_buffer_t *buffer;
-};
-
-static void
-on_timeout (ply_boot_splash_t *splash)
-{
-  ply_boot_splash_update_status (splash, "foo");
-  ply_event_loop_watch_for_timeout (splash->loop, 
-                                    5.0,
-                                   (ply_event_loop_timeout_handler_t)
-                                   on_timeout,
-                                   splash);
-}
-
-static void
-on_quit (test_state_t *state)
-{
-    ply_boot_splash_hide (state->splash);
-    ply_event_loop_exit (state->loop, 0);
-}
-
-static void
-add_displays_to_splash_from_renderer (test_state_t   *state,
-                                      ply_renderer_t *renderer)
-{
-  ply_list_t *heads;
-  ply_list_node_t *node;
-
-  heads = ply_renderer_get_heads (renderer);
-
-  node = ply_list_get_first_node (heads);
-  while (node != NULL)
-    {
-      ply_list_node_t *next_node;
-      ply_renderer_head_t *head;
-      ply_pixel_display_t *display;
-
-      head = ply_list_node_get_data (node);
-      next_node = ply_list_get_next_node (heads, node);
-
-      display = ply_pixel_display_new (renderer, head);
-
-      ply_boot_splash_add_pixel_display (state->splash, display);
-
-      node = next_node;
-    }
-}
-
-int
-main (int    argc,
-      char **argv)
-{
-  int exit_code;
-  test_state_t state;
-  char *tty_name;
-  const char *theme_path;
-  ply_text_display_t *text_display;
-  ply_renderer_t *renderer;
-  ply_terminal_t *terminal;
-  ply_keyboard_t *keyboard;
-
-  exit_code = 0;
-
-  state.loop = ply_event_loop_new ();
-
-  if (argc > 1)
-    theme_path = argv[1];
-  else
-    theme_path = PLYMOUTH_THEME_PATH "/fade-in/fade-in.plymouth";
-
-  if (argc > 2)
-    asprintf(&tty_name, "tty%s", argv[2]);
-  else
-    tty_name = strdup("tty0");
-
-  terminal = ply_terminal_new (tty_name);
-
-  if (!ply_terminal_open (terminal))
-    {
-      perror ("could not open tty");
-      return errno;
-    }
-
-  renderer = ply_renderer_new (NULL, terminal);
-  free(tty_name);
-
-  if (!ply_renderer_open (renderer))
-    {
-      perror ("could not open renderer /dev/fb");
-      ply_renderer_free (renderer);
-      return errno;
-    }
-
-  keyboard = ply_keyboard_new_for_renderer (renderer);
-  ply_keyboard_add_escape_handler (keyboard,
-                                   (ply_keyboard_escape_handler_t) on_quit, &state);
-
-  state.buffer = ply_buffer_new ();
-  state.splash = ply_boot_splash_new (theme_path, PLYMOUTH_PLUGIN_PATH, state.buffer);
-
-  if (!ply_boot_splash_load (state.splash))
-    {
-      perror ("could not load splash screen");
-      return errno;
-    }
-
-  ply_boot_splash_set_keyboard (state.splash, keyboard);
-  add_displays_to_splash_from_renderer (&state, renderer);
-
-  text_display = ply_text_display_new (terminal);
-  ply_boot_splash_add_text_display (state.splash, text_display);
-
-  ply_boot_splash_attach_to_event_loop (state.splash, state.loop);
-
-  if (!ply_boot_splash_show (state.splash, PLY_BOOT_SPLASH_MODE_BOOT_UP))
-    {
-      perror ("could not show splash screen");
-      return errno;
-    }
-
-  ply_event_loop_watch_for_timeout (state.loop, 
-                                    1.0,
-                                   (ply_event_loop_timeout_handler_t)
-                                   on_timeout,
-                                   state.splash);
-  exit_code = ply_event_loop_run (state.loop);
-  ply_boot_splash_free (state.splash);
-  ply_buffer_free (state.buffer);
-
-  return exit_code;
-}
-
-#endif /* PLY_BOOT_SPLASH_ENABLE_TEST */
 /* vim: set ts=4 sw=4 expandtab autoindent cindent cino={.5s,(0: */
